@@ -1,13 +1,12 @@
 from PyQt4 import QtCore, QtGui, QtSql
 from ui_settings import Ui_Settings
 from ui_login import Ui_Login
-from ui_tags import Ui_Tags
 from ui_import import Ui_Import
 import enum
-from utils import showWaitCursor
+import utils
 from database import db, DatabaseNotInitialisedException, ConnectionException
 from delegates import AccountDelegate
-from models import AccountModel, TagModel, ImportModel
+from models import AccountModel, ImportModel
 
 class UserCancelledException(Exception):
 	""" Exception to indicate user has cancelled the current operation
@@ -23,8 +22,7 @@ class ImportDialog(Ui_Import, QtGui.QDialog):
 		self.__cancelImport = False
 
 		self.progressBar.setVisible(False)
-		model = ImportModel(self)
-		model.loadRecords(records)
+		model = ImportModel(records, self)
 		db.commit.connect(model.save)
 		db.rollback.connect(model.reset)
 
@@ -42,12 +40,15 @@ class ImportDialog(Ui_Import, QtGui.QDialog):
 		self.view.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
 		self.view.resizeColumnsToContents()
 		self.view.horizontalHeader().setStretchLastSection(True)
-		self.view.sortByColumn(1, QtCore.Qt.AscendingOrder)
+		self.view.sortByColumn(2, QtCore.Qt.AscendingOrder)
+		self.view.resizeColumnsToContents()
 
 		self.closeButton.clicked.connect(self.__close)
 		self.view.selectionModel().selectionChanged.connect(self.__recordsSelected)
 
 		self.importCancelButton.setEnabled(False)
+		
+		self.selectAllButton.setEnabled(bool(model.numRecordsToImport()))
 
 		self.__setCounters()
 
@@ -59,26 +60,16 @@ class ImportDialog(Ui_Import, QtGui.QDialog):
 
 	def __setCounters(self):
 		model = self.view.model().sourceModel()
-		self.errorsCounter.setNum(model.numBadRecords)
-		self.importedCounter.setNum(model.numRecordsImported)
-		self.toImportCounter.setNum(model.numRecordsToImport)
+		self.errorsCounter.setNum(model.numBadRecords())
+		self.importedCounter.setNum(model.numRecordsImported())
+		self.toImportCounter.setNum(model.numRecordsToImport())
 
 	def __recordsSelected(self):
-		selectionModel = self.view.selectionModel()
-		proxyModel = self.view.model()
-		dataModel = proxyModel.sourceModel()
-
-		selectionModel.blockSignals(True)
-		for index in selectionModel.selectedRows():
-			# de-select any that have errors or duplicates
-			if not dataModel.canImport(proxyModel.mapToSource(index)):
-				selectionModel.select(index, QtGui.QItemSelectionModel.Deselect | QtGui.QItemSelectionModel.Rows)
-
-		# get the new selection
-		numSelected = len(selectionModel.selectedRows())
+		""" Enable button cancel when we have selection
+		"""
+		numSelected = len(self.view.selectionModel().selectedRows())
 		self.selectedCounter.setNum(numSelected)
 		self.importCancelButton.setEnabled(bool(numSelected))
-		selectionModel.blockSignals(False)
 
 	def __close(self):
 		self.done(self.view.model().sourceModel().dataSaved)
@@ -102,8 +93,8 @@ class ImportDialog(Ui_Import, QtGui.QDialog):
 			self.__importInProgress = True
 			self.closeButton.setEnabled(False)
 			self.selectAllButton.setEnabled(False)
-			self.importCancelButton.setEnabled(True)
 			self.importCancelButton.setText('Cancel')
+			self.importCancelButton.setEnabled(True)
 			
 			# Wrap the import in a transaction
 			with db.transaction():
@@ -125,8 +116,6 @@ class ImportDialog(Ui_Import, QtGui.QDialog):
 						# By raising here we will rollback the database transaction
 						raise UserCancelledException
 
-			self.importCancelButton.setEnabled(bool(dataModel.numRecordsToImport))
-
 		except UserCancelledException:
 			dataModel.reset()
 
@@ -137,11 +126,13 @@ class ImportDialog(Ui_Import, QtGui.QDialog):
 			self.__cancelImport = False
 			self.__importInProgress = False
 			self.closeButton.setEnabled(True)
-			self.importCancelButton.setEnabled(False)
-			self.selectAllButton.setEnabled(True)
 			self.importCancelButton.setText('Import')
 			self.progressBar.setVisible(False)
 
+			canImport = bool(dataModel.numRecordsToImport())
+			self.importCancelButton.setEnabled(canImport)
+			self.selectAllButton.setEnabled(canImport)
+
 
 class SettingsDialog(Ui_Settings, QtGui.QDialog):
 	def __init__(self, userId, parent=None):
@@ -171,7 +162,7 @@ class SettingsDialog(Ui_Settings, QtGui.QDialog):
 		self.view.sortByColumn(enum.kAccountTypeColumn_AccountName, QtCore.Qt.AscendingOrder)
 		self.view.resizeColumnsToContents()
 		self.view.horizontalHeader().setStretchLastSection(True)
-		self.view.setItemDelegate(AccountDelegate())
+		self.view.setItemDelegate(AccountDelegate(self))
 
 		model.dataChanged.connect(self.view.clearSelection)
 		model.dataChanged.connect(self.__dataChanged)
@@ -183,26 +174,40 @@ class SettingsDialog(Ui_Settings, QtGui.QDialog):
 		self.enableCommit(True)
 
 	def validateNewAccount(self, record):
-		error = ''
-		if not record.value(enum.kAccountTypeColumn_AccountName).toString():
-			error = "Account name cannot be empty!"
-		elif record.value(enum.kAccountTypeColumn_DateField).isNull():
-			error = "Date field must be set!"
-		elif record.value(enum.kAccountTypeColumn_DescriptionField).isNull():
-			error = "Description field must be set!"
-		elif record.value(enum.kAccountTypeColumn_CurrencySign).toPyObject() not in (1, -1):
-			error = "Current sign value must be 1 or -1"
-		elif not record.value(enum.kAccountTypeColumn_DateFormat).toString():
-			error = "Date format cannot be empty!"
+		try:
+			if record.value(enum.kAccountTypeColumn_AccountName).toString().isEmpty():
+				raise Exception('Account name cannot be empty')
 
-		if error:
-			QtGui.QMessageBox.critical(self, 'Account failed', error, QtGui.QMessageBox.Ok)
+			value, ok = record.value(enum.kAccountTypeColumn_DateField).toInt()
+			if not ok or value < 0:
+				raise Exception('Date field must be set (index of date field)')
+
+			value, ok = record.value(enum.kAccountTypeColumn_CreditField).toInt()
+			if not ok or value < 0:
+				raise Exception('Credit field must be set (index of credit field)')
+
+			value, ok = record.value(enum.kAccountTypeColumn_DebitField).toInt()
+			if not ok or value < 0:
+				raise Exception('Debit field must be set (index of debit field)')
+
+			if record.value(enum.kAccountTypeColumn_DescriptionField).toString().isEmpty():
+				raise Exception('Description field must be set (index of description field)')
+
+			value, ok = record.value(enum.kAccountTypeColumn_CurrencySign).toInt()
+			if not ok or value not in (1, -1):
+				raise Exception('Currency sign value must be 1 or -1')
+
+			if record.value(enum.kAccountTypeColumn_DateFormat).toString().isEmpty():
+				raise Exception('"Date format cannot be empty (eg "dd/MM/yyyy")')
+
+		except Exception, err:
+			QtGui.QMessageBox.critical(self, 'Account failed', str(err), QtGui.QMessageBox.Ok)
 			# Trash the bad record
 			record.clear()
 
 	def saveSettings(self):
 
-		if self.model.submitAll() and self.model.lastError().isValid():
+		if not self.model.submitAll() and self.model.lastError().isValid():
 			# If we've cleared the record from validateNewAccount() then the database error
 			# will be empty. No need to issue a second error message
 			if self.m_model.lastError().databaseText():
@@ -223,7 +228,11 @@ class SettingsDialog(Ui_Settings, QtGui.QDialog):
 		rowCount = self.model.rowCount()
 		self.model.insertRow(rowCount)
 
-		index = self.model.index(rowCount, 1)
+		for column in xrange(1, self.model.columnCount()):
+			index = self.model.index(rowCount, column)
+			self.model.setData(index, QtCore.QVariant(), QtCore.Qt.EditRole)
+
+		index = self.model.index(rowCount, enum.kAccountTypeColumn_AccountName)
 		self.view.setCurrentIndex(index)
 		self.view.edit(index)
 
@@ -262,7 +271,7 @@ class LoginDialog(Ui_Login, QtGui.QDialog):
 		db.port = self.portSpinBox.value()
 
 		try:
-			with showWaitCursor():
+			with utils.showWaitCursor():
 				db.connect()
 		except DatabaseNotInitialisedException:
 			if QtGui.QMessageBox.question(
@@ -281,53 +290,3 @@ class LoginDialog(Ui_Login, QtGui.QDialog):
 			QtGui.QMessageBox.warning(self, 'Database', 'Failed to connect: %s' % str(err))
 		else:
 			self.accept()
-
-class TagDialog(Ui_Tags, QtGui.QDialog):
-
-	# Emitted when the TagModel data has changed (tag added or removed)
-	dataChanged= QtCore.pyqtSignal()
-
-	def __init__(self, recordIds, parent=None):
-		super(TagDialog, self).__init__(parent=parent)
-
-		self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
-		self.setupUi(self)
-
-		self.tagView.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
-		self.tagView.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
-		self.tagView.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
-		self.deleteTagButton.setEnabled(False)
-
-		model = TagModel(recordIds, self)
-		self.tagView.setModel(model)
-		self.tagView.setModelColumn(enum.kTagsColumn_TagName)
-
-		self.addTagButton.pressed.connect(self.addTag)
-		self.deleteTagButton.pressed.connect(self.deleteTags)
-		self.tagView.selectionModel().selectionChanged.connect(self.activateDeleteTagButton)
-
-		self.model = model
-
-	def activateDeleteTagButton(self, selected):
-		self.deleteTagButton.setEnabled(len(selected) > 0)
-
-	def addTag(self):
-		""" Add a new tag
-		"""
-		tagName, ok = QtGui.QInputDialog.getText(self, 'New Tag', 'Tag', QtGui.QLineEdit.Normal)
-
-		if ok and tagName:
-			if tagName in self.model:
-				QtGui.QMessageBox.critical( self, 'Tag Error', 'Tag already exists!', QtGui.QMessageBox.Ok)
-				return
-
-			self.model.addTag(tagName)
-			self.dataChanged.emit()
-
-	def deleteTags(self):
-		rows = self.tagView.selectionModel().selectedRows()
-		if rows:
-			for index in reversed(rows):
-				self.model.removeRows(index.row(), 1, QtCore.QModelIndex())
-			self.dataChanged.emit()
-
